@@ -1,6 +1,6 @@
 /*
 
-(C) 2009-2015 Mika Ilmaranta <ilmis@nullnet.fi>
+(C) 2009-2019 Mika Ilmaranta <ilmis@nullnet.fi>
 
 License: GPLv2
 
@@ -18,11 +18,14 @@ License: GPLv2
 #include <unistd.h>
 
 #include <dirent.h>
+#include <linux/limits.h>
 #include <fnmatch.h>
 
 #include "config.h"
 #include "defs.h"
 
+#define DEFAULT_SCRIPT_FILE SCRIPTDIR "/default_script"
+
 static CONFIG defaults;
 static int errors = 0;
 
@@ -108,6 +111,7 @@ void free_config(CONFIG **first, CONFIG **last, GROUPS **firstg, GROUPS **lastg)
 		if(curg->eventscript && curg->eventscript != defaults.eventscript)	release(&curg->eventscript);
 		if(curg->notifyscript && curg->notifyscript != defaults.notifyscript)	release(&curg->notifyscript);
 		if(curg->warn_email && curg->warn_email != defaults.warn_email)		release(&curg->warn_email);
+		if(curg->device && curg->device != defaults.device)                     release(&curg->device);
 		if(curg->queue && curg->queue != defaults.queue)                        release(&curg->queue);
 
 		prevg = curg;
@@ -144,7 +148,7 @@ void init_config(void)
 	defaults.name = strdup("defaults");
 	defaults.checkip = strdup("127.0.0.1");
 	defaults.eventscript = NULL;
-	defaults.notifyscript = strdup("/usr/libexec/lsm/default_script");
+	defaults.notifyscript = strdup(DEFAULT_SCRIPT_FILE);
 	defaults.max_packet_loss = 15;
 	defaults.max_successive_pkts_lost = 7;
 	defaults.min_packet_loss = 5;
@@ -178,7 +182,7 @@ void init_config(void)
 static int find_all_configs(char* fn, int mustexist, CONFIG **first, CONFIG **last, GROUPS **firstg, GROUPS **lastg)
 {
 	struct dirent **namelist;
-	char dir[BUFSIZ], pattern[128], *p, s[BUFSIZ];
+	char dir[PATH_MAX - NAME_MAX - 1], pattern[128], *p, s[PATH_MAX];
 	int n, i, found;
 
 	/* Split fn to dir/pattern */
@@ -205,7 +209,7 @@ static int find_all_configs(char* fn, int mustexist, CONFIG **first, CONFIG **la
 	for (i = 0; i < n; i++) {
 		if (fnmatch(pattern, namelist[i]->d_name, 0) == 0 &&
 		    fnmatch("*~", namelist[i]->d_name, 0) != 0) {
-			snprintf(s, BUFSIZ, "%s/%s", dir, namelist[i]->d_name);
+			snprintf(s, PATH_MAX, "%s/%s", dir, namelist[i]->d_name);
 			read_one_config(s, first, last, firstg, lastg);
 			found++;
 		}
@@ -424,6 +428,7 @@ static void read_one_config(char *fn, CONFIG **first, CONFIG **last, GROUPS **fi
 				else if(!eqcmp(buf, "unknown_up_notify"))          curg->unknown_up_notify              = atoi(strchr(buf, '=') + 1);
 				else if(!eqcmp(buf, "warn_email"))                 curg->warn_email			= strdup(strchr(buf, '=') + 1);
 				else if(!eqcmp(buf, "logic"))                      curg->logic				= atoi(strchr(buf, '=') + 1);
+				else if(!eqcmp(buf, "device"))                     curg->device                         = strdup(strchr(buf, '=') + 1);
 				else if(!eqcmp(buf, "status"))			   curg->status                         = atoi(strchr(buf, '=') + 1);
 				else if(!eqcmp(buf, "queue"))                      curg->queue                          = strdup(strchr(buf, '=') + 1);
 				else if(!eqcmp(buf, "member-connection")) {
@@ -533,6 +538,7 @@ static void read_one_config(char *fn, CONFIG **first, CONFIG **last, GROUPS **fi
 				curg->unknown_up_notify = defaults.unknown_up_notify;
 				curg->warn_email      = defaults.warn_email;
 				curg->logic           = 0; /* default group logic or */
+				curg->device          = defaults.device;
 				curg->status          = defaults.status;
 				curg->queue           = defaults.queue;
 
@@ -635,6 +641,7 @@ void dump_config(CONFIG **first, CONFIG **last, GROUPS **firstg, GROUPS **lastg)
 		syslog(LOG_INFO, "curg->notifyscript            = \"%s\"", curg->notifyscript);
 		syslog(LOG_INFO, "curg->unknown_up_notify       = \"%d\"", curg->unknown_up_notify);
 		syslog(LOG_INFO, "curg->warn_email              = \"%s\"", curg->warn_email);
+		syslog(LOG_INFO, "curg->device                  = \"%s\"", curg->device);
 		syslog(LOG_INFO, "curg->logic                   = \"%s\"", curg->logic == 0 ? "OR" : "AND");
 
 		for(curgm = curg->fgm; curgm; curgm = curgm->next) {
